@@ -55,8 +55,8 @@ def preprocess_for_parallel_calculations(parameter_array_all, task_num=1, task_i
             parameter_array = parameter_array_all[task_index*num_parameter:num_all]
     return parameter_array
 
-# 创建一个sh文件用于提交任务
-def make_sh_file(sh_filename='a', command_line='python a.py', cpu_num=1, task_name='task', cd_dir=0):
+# 创建一个sh文件用于提交任务（PBS）
+def make_sh_file_for_qsub(sh_filename='a', command_line='python a.py', cpu_num=1, task_name='task', cd_dir=0):
     sh_content = \
         '#!/bin/sh\n' \
         +'#PBS -N '+task_name+'\n' \
@@ -67,7 +67,20 @@ def make_sh_file(sh_filename='a', command_line='python a.py', cpu_num=1, task_na
     with open(sh_filename+'.sh', 'w') as f:
         f.write(sh_content)
 
-# 复制.py和.sh文件，然后提交任务，实现半手动并行
+# 创建一个sh文件用于提交任务（LSF）
+def make_sh_file_for_bsub(sh_filename='a', command_line='python a.py', cpu_num=1, task_name='task', queue_name='score', cd_dir=0):
+    sh_content = \
+        '#!/bin/sh\n' \
+        +'#BSUB -J '+task_name+'\n' \
+        +'#BSUB -q '+queue_name+'\n' \
+        +'#BSUB -n '+str(cpu_num)+'\n'
+    if cd_dir==1:
+        sh_content += 'cd $PBS_O_WORKDIR\n'
+    sh_content += command_line
+    with open(sh_filename+'.sh', 'w') as f:
+        f.write(sh_content)
+
+# 复制.py和.sh文件，然后提交任务，实现半手动并行（PBS）
 def copy_py_sh_file_and_qsub_task(parameter_array, py_filename='a', old_str_in_py='parameter=0', new_str_in_py='parameter=', sh_filename='a', qsub_task_name='task'):
     import os
     parameter_str_array = []
@@ -104,6 +117,43 @@ def copy_py_sh_file_and_qsub_task(parameter_array, py_filename='a', old_str_in_p
         # qsub task
         os.system('qsub '+new_file)
 
+# 复制.py和.sh文件，然后提交任务，实现半手动并行（LSF）
+def copy_py_sh_file_and_bsub_task(parameter_array, py_filename='a', old_str_in_py='parameter=0', new_str_in_py='parameter=', sh_filename='a', bsub_task_name='task'):
+    import os
+    parameter_str_array = []
+    for i0 in parameter_array:
+        parameter_str_array.append(str(i0))
+    index = 0
+    for parameter_str in parameter_str_array:
+        index += 1
+        # copy python file
+        old_file = py_filename+'.py'
+        new_file = py_filename+'_'+str(index)+'.py'
+        os.system('cp '+old_file+' '+new_file)
+        with open(new_file, 'r') as f:
+            content  = f.read()
+        old_str = old_str_in_py
+        new_str = new_str_in_py+parameter_str
+        content = content.replace(old_str, new_str)
+        with open(py_filename+'_'+str(index)+'.py', 'w') as f:
+            f.write(content)
+        # copy sh file
+        old_file = sh_filename+'.sh'
+        new_file = sh_filename+'_'+str(index)+'.sh'
+        os.system('cp '+old_file+' '+new_file)
+        with open(new_file, 'r') as f:
+            content  = f.read()
+        old_str = 'python '+py_filename+'.py'
+        new_str = 'python '+py_filename+'_'+str(index)+'.py'
+        content = content.replace(old_str, new_str)
+        old_str = bsub_task_name
+        new_str = bsub_task_name+'_'+str(index)
+        content = content.replace(old_str, new_str)
+        with open(sh_filename+'_'+str(index)+'.sh', 'w') as f: 
+            f.write(content)
+        # bsub task
+        os.system('bsub < '+new_file)
+
 # 自动先后运行程序
 def run_programs_sequentially(program_files=['./a.py', './b.py'], execute='python ', show_time=0):
     import os
@@ -123,6 +173,62 @@ def run_programs_sequentially(program_files=['./a.py', './b.py'], execute='pytho
         end = time.time()
         print('Total running time = '+str((end-start)/60)+' min')
 
+# 将XYZ数据转成矩阵数据（说明：x_array/y_array的输入和输出不一样。要求z_array数据中y对应的数据为小循环，x对应的数据为大循环）
+def convert_xyz_data_into_matrix_data(x_array, y_array, z_array):
+    import numpy as np
+    x_array_input = np.array(x_array)
+    y_array_input = np.array(y_array)
+    x_array = np.array(list(set(x_array_input)))
+    y_array = np.array(list(set(y_array_input)))
+    z_array = np.array(z_array)
+    len_x = len(x_array)
+    len_y = len(y_array)
+    matrix = np.zeros((len_x, len_y))
+    for ix in range(len_x):
+        for iy in range(len_y):
+            matrix[ix, iy] = z_array[ix*len_y+iy]
+    return x_array, y_array, matrix
+
+# 将矩阵数据转成XYZ数据（说明：x_array/y_array的输入和输出不一样。生成的z_array数据中y对应的数据为小循环，x对应的数据为大循环）
+def convert_matrix_data_into_xyz_data(x_array, y_array, matrix):
+    import numpy as np
+    x_array_input = np.array(x_array)
+    y_array_input = np.array(y_array)
+    matrix = np.array(matrix)
+    len_x = len(x_array_input)
+    len_y = len(y_array_input)
+    x_array = np.zeros((len_x*len_y))
+    y_array = np.zeros((len_x*len_y))
+    z_array = np.zeros((len_x*len_y))
+    for ix in range(len_x):
+        for iy in range(len_y):
+            x_array[ix*len_y+iy] = x_array_input[ix]
+            y_array[ix*len_y+iy] = y_array_input[iy]
+            z_array[ix*len_y+iy] = matrix[ix, iy]
+    return x_array, y_array, z_array
+
+# 通过定义计算R^2（基于实际值和预测值）
+def calculate_R2_with_definition(y_true_array, y_pred_array):
+    import numpy as np
+    y_mean = np.mean(y_true_array)
+    SS_tot = np.sum((y_true_array - y_mean) ** 2)
+    SS_res = np.sum((y_true_array - y_pred_array) ** 2)
+    R2 = 1 - (SS_res / SS_tot)
+    return R2
+
+# 通过sklearn计算R^2，和定义的计算结果一致
+def calculate_R2_with_sklearn(y_true_array, y_pred_array):
+    from sklearn.metrics import r2_score
+    R2 = r2_score(y_true_array, y_pred_array)
+    return R2
+
+# 通过scipy计算线性回归后的R^2（基于线性回归模型）
+def calculate_R2_after_linear_regression_with_scipy(y_true_array, y_pred_array):
+    from scipy import stats
+    slope, intercept, r_value, p_value, std_err = stats.linregress(y_true_array, y_pred_array)
+    R2 = r_value**2
+    return R2
+
 # 获取函数或类的源码（返回字符串）
 def get_source(name):
     import inspect
